@@ -1,10 +1,10 @@
-from .hmm import *
 from .base import *
+from .hmm import *
 from .mc import *
-from .gohmm import *
 from .mdp import *
 from .ctmc import *
-from .mgohmm import *
+from .gohmm import *
+from .pctmc import *
 try:
 	from .with_stormpy import *
 except ModuleNotFoundError:
