@@ -3,19 +3,22 @@ from ast import literal_eval
 from ..base.tools import resolveRandom, randomProbabilities
 from ..base.Set import Set
 from ..mc.MC import MC
-from ..base.Model import Model
+from ..pctmc.PCTMC import PCTMC
+from ..base.Base_MC import *
 from math import exp, log
 from random import randint
-from numpy import array, zeros, dot, ndarray, vstack, hstack, newaxis, append
+from numpy import array, zeros, dot, ndarray, vstack, hstack, newaxis, append, delete, where, insert
 from sys import platform
 from multiprocessing import cpu_count, Pool
-from random import choices
+from sympy import sympify
 
-class CTMC(Model):
+class CTMC(Base_MC):
 	"""
 	Class representing a CTMC.
 	"""
-	def __init__(self, matrix: ndarray, labeling: list, name: str ="unknown_CTMC") -> None:
+	def __init__(self, matrix: ndarray, labelling: list,
+				 name: str ="unknown_CTMC",
+				 synchronous_transitions: list =[]) -> None:
 		"""
 		Creates an CTMC.
 
@@ -26,33 +29,27 @@ class CTMC(Model):
 			Represents the transition matrix.
 			`matrix[s1][s2]` is the rate associated to the transition 
 			from `s1` to `s2`.
-		labeling: list of str
+		labelling: list of str
 			A list of N observations (with N the nb of states).
-			If `labeling[s] == o` then state of ID `s` is labelled by `o`.
+			If `labelling[s] == o` then state of ID `s` is labelled by `o`.
 			Each state has exactly one label.
-		initial_state : int or list of float
-			Determine which state is the initial one (then it's the id of the
-			state), or what are the probability to start in each state (then it's
-			a list of probabilities).
 		name : str, optional
 			Name of the model.
 			Default is "unknow_CTMC"
+		synchronous_transitions: list, optional.
+			This is useful only for synchronously composing this CTMC with
+			another one.
+			List of (source_state <int>, action <str>, dest_state <int>, rate <float>).
+			Default is an empty list.
 		"""
-		self.alphabet = list(set(labeling))
-		self.labeling = labeling
+		self.model_type = CTMC_ID
+		self.synchronous_transitions = synchronous_transitions
 
-		if not 'init' in self.labeling:
-			msg = "No initial state given: at least one"
-			msg += " state should be labelled by 'init'."
-			raise ValueError(msg)
-		initial_state = [1.0/self.labeling.count("init") if i=='init' else 0.0 for i in self.labeling]
-
-		super().__init__(matrix,initial_state,name)
-		if len(labeling) != self.nb_states:
-			raise ValueError("The length of labeling is not equal to the number of states")
+		super().__init__(matrix,labelling,name)
 		for s in range(self.nb_states):
-			if self.e(s) == 0.0:
-				raise ValueError("State "+str(s)+" doesn't have any leaving transition.")
+			synchronous_transitions_source = [i[0] for i in synchronous_transitions]
+			if self.e(s) == 0.0 and s not in synchronous_transitions_source:
+				print("WARNING: State "+str(s)+" doesn't have any leaving transition.")
 
 	def e(self,s: int) -> float:
 		"""
@@ -90,7 +87,7 @@ class CTMC(Model):
 		"""
 		self._checkStateIndex(s1)
 		self._checkStateIndex(s2)
-		if self.labeling[s1] != obs:
+		if self.labelling[s1] != obs:
 			return 0.0
 		return self.matrix[s1][s2]
 	
@@ -152,46 +149,8 @@ class CTMC(Model):
 		"""
 		return 1/self.e(s)
 
-	def getLabel(self,state: int) -> str:
-		"""
-		Returns the label of `state`.
-
-		Parameters
-		----------
-		state : int
-			a state ID
-
-		Returns
-		-------
-		str
-			a label
-
-		Example
-		-------
-		>>> model.getLabel(2)
-		'Label-of-state-2'
-		"""
-		self._checkStateIndex(state)
-		return self.labeling[state]
-	
-	def getAlphabet(self) -> list:
-		"""
-		Returns the alphabet of this model.
-
-		Returns
-		-------
-		list of str
-			The alphabet of this model
-		
-		Example
-		-------
-		>>> model.getAlphabet()
-		['a','b','c','d','done']
-		"""
-		return self.alphabet
-
 	def _stateToString(self,state:int) -> str:
-		res = "----STATE "+str(state)+"--"+self.labeling[state]+"----\n"
+		res = "----STATE "+str(state)+"--"+self.labelling[state]+"----\n"
 		res += "Exepected waiting time: "+str(self.expected_time(state))+'\n'
 		den = self.e(state)
 		for j in range(len(self.matrix[state])):
@@ -217,7 +176,7 @@ class CTMC(Model):
 			else:
 				exps.append(exponential(1/exp_lambda))
 		next_state= exps.index(min(exps))
-		next_obs = self.labeling[state]
+		next_obs = self.labelling[state]
 		return (next_obs, next_state, min(exps))
 
 	def run(self,number_steps: int, timed: bool = False) -> list:
@@ -248,7 +207,7 @@ class CTMC(Model):
 				output.append(time_spent)
 			current = next_state
 			c += 1
-		output.append(self.labeling[current])
+		output.append(self.labelling[current])
 		return output
 	
 	def _computeAlphas_timed(self, sequence: list, times: int) -> float:
@@ -262,11 +221,27 @@ class CTMC(Model):
 				p = array([self.l(ss,s,obs_seq[k])*exp(-self.e(ss)*times_seq[k]) for ss in range(self.nb_states) ])
 				new_arr[s] = dot(prev_arr,p)
 			prev_arr = new_arr
-		last_arr = prev_arr * (array(self.labeling) == obs_seq[-1])
-		return log(last_arr.sum())*times
-
+		last_arr = prev_arr * (array(self.labelling) == obs_seq[-1])
+		try:
+			res = log(last_arr.sum())*times
+		except ValueError:
+			return False
+		return res
 
 	def logLikelihood(self,traces: Set) -> float:
+		"""
+		Computes the loglikelihood of `traces` under this model.
+
+		Parameters
+		----------
+		traces : Set
+			a set of traces.
+
+		Returns
+		-------
+		float
+			the loglikelihood of `traces` under this model.
+		"""
 		if traces.type == 0: # non-timed traces
 			return super().logLikelihood(traces)
 		else: # timed traces
@@ -278,6 +253,10 @@ class CTMC(Model):
 				temp = [res.get() for res in tasks if res.get() != False]
 			else:
 				temp = [self._computeAlphas_timed(seq,times) for seq,times in zip(traces.sequences,traces.times)]
+				while False in temp:
+					temp.remove(False)
+			if len(temp) == 0:
+				return -9999
 			return sum(temp)/sum(traces.times)
 
 	def toMC(self, name: str="unknown_MC") -> MC:
@@ -298,9 +277,9 @@ class CTMC(Model):
 		for i in range(self.nb_states):
 			new_matrix[i] /= self.e(i)
 
-		return MC(new_matrix,self.labeling,name)
+		return MC(new_matrix,self.labelling,name)
 
-	def save(self,file_path:str):
+	def save(self,file_path:str) -> None:
 		"""Save the model into a text file.
 
 		Parameters
@@ -314,11 +293,31 @@ class CTMC(Model):
 		"""
 		f = open(file_path, 'w')
 		f.write("CTMC\n")
-		f.write(str(self.labeling))
-		f.write('\n')
 		super()._save(f)
+	
+	def savePrism(self,file_path:str) -> None:
+		"""
+		Save this model into `file_path` in the Prism format.
 
-def loadCTMC(file_path: str) -> MC:
+		Parameters
+		----------
+		file_path : str
+			Path of the output file.
+		"""
+		f = open(file_path,'w')
+		f.write("ctmc\n\n")
+		super()._savePrism(f)
+	
+	def _savePrismMatrix(self,f) -> None:
+		for s1 in range(self.nb_states):
+			if (self.matrix[s1]!=0).any():
+				res = '\t[] s='+str(s1)+' ->'
+				for s2 in where(self.matrix[s1] != 0.0)[0]:
+					res += ' '+str(self.matrix[s1,s2])+":(s'="+str(s2)+") +"
+				res = res[:-2]+';\n'
+				f.write(res)
+
+def loadCTMC(file_path: str) -> CTMC:
 	"""
 	Load an CTMC saved into a text file.
 
@@ -336,15 +335,15 @@ def loadCTMC(file_path: str) -> MC:
 	l = f.readline()[:-1] 
 	if l != "CTMC":
 		print("ERROR: this file doesn't describe an CTMC: it describes a "+l)
-	labeling = literal_eval(f.readline()[:-1])
+	labelling = literal_eval(f.readline()[:-1])
 	name = f.readline()[:-1]
 	initial_state = array(literal_eval(f.readline()[:-1]))
 	matrix = literal_eval(f.readline()[:-1])
 	matrix = array(matrix)
 	f.close()
-	return CTMC(matrix, labeling, name)
+	return CTMC(matrix, labelling, name)
 
-def CTMC_random(nb_states: int, alphabet: list, min_exit_rate_time : int,
+def CTMC_random(nb_states: int, labelling: list, min_exit_rate_time : int,
 				max_exit_rate_time: int, self_loop: bool = True,
 				random_initial_state: bool=True) -> CTMC:
 	"""
@@ -355,7 +354,7 @@ def CTMC_random(nb_states: int, alphabet: list, min_exit_rate_time : int,
 	----------
 	nb_states : int
 		Number of states.
-	alphabet : list of str
+	labelling : list of str
 		List of observations.
 	min_exit_rate_time: int
 		Minimum exit rate for the states (included).
@@ -392,40 +391,29 @@ def CTMC_random(nb_states: int, alphabet: list, min_exit_rate_time : int,
 	s2 -> s0 : lambda = 0.2
 	s2 -> s1 : lambda = 0.8
 	"""
-	if nb_states < len(alphabet):
-		print("WARNING: the size of the alphabet is higher than the",end=" ")
-		print("number of states. Some labels will not be assigned to",end=" ")
-		print("any states.")
-	
-	if 'init' in alphabet:
-		msg =  "The label 'init' cannot be used: it is reserved for initial states."
-		raise SyntaxError(msg)
-
-	
-	labeling = alphabet[:min(len(alphabet),nb_states)] + choices(alphabet,k=nb_states-len(alphabet))
-	
-	matrix = []
+	matrix = zeros((nb_states+1,nb_states))
 	for i in range(nb_states):
 		if self_loop:
-			random_probs = array(randomProbabilities(nb_states))
+			matrix[i] = array(randomProbabilities(nb_states))
 		else:
-			p = randomProbabilities(nb_states-1)
-			random_probs.insert(i,0.0)
-		p = randomProbabilities(nb_states)
+			p = randomProbabilities(nb_states-1).tolist()
+			p = insert(p,i,0.0)
+			matrix[i] = array(p)
 		av_waiting_time = randint(min_exit_rate_time,max_exit_rate_time)
-		p = random_probs/av_waiting_time
-		matrix.append(p)
-	
-	labeling.append("init")
-	if random_initial_state:
-		matrix.append(append(randomProbabilities(nb_states),0.0))
-	else:
-		matrix.append([1.0]+[0.0 for i in range(nb_states)])
-	
-	matrix = array(matrix)
-	return CTMC(matrix, labeling,"CTMC_random_"+str(nb_states)+"_states")
+		matrix[i] /= av_waiting_time
 
-def createCTMC(transitions: list, labeling: list, initial_state, name: str ="unknown_CTMC") -> CTMC:
+	if random_initial_state:
+		matrix[-1] = randomProbabilities(nb_states)
+	else:
+		matrix[-1] = array([1.0]+[0.0 for i in range(nb_states-1)])
+	
+	matrix = hstack((matrix,zeros(len(matrix))[:,newaxis]))
+	
+	labelling = labelsForRandomModel(nb_states,labelling)
+	return CTMC(matrix, labelling,"CTMC_random_"+str(nb_states)+"_states")
+
+def createCTMC(transitions: list, labelling: list, initial_state,
+			   name: str ="unknown_CTMC",synchronous_transitions: list =[]) -> CTMC:
 	"""
 	An user-friendly way to create a CTMC.
 
@@ -434,9 +422,9 @@ def createCTMC(transitions: list, labeling: list, initial_state, name: str ="unk
 	transitions : [ list of tuples (int, int, float)]
 		Each tuple represents a transition as follow: 
 		(source state ID, destination state ID, rate).
-	labeling: list of str
+	labelling: list of str
 		A list of N observations (with N the nb of states).
-		If `labeling[s] == o` then state of ID `s` is labelled by `o`.
+		If `labelling[s] == o` then state of ID `s` is labelled by `o`.
 		Each state has exactly one label.
 	initial_state : int or list of float
 		Determine which state is the initial one (then it's the id of the
@@ -445,11 +433,16 @@ def createCTMC(transitions: list, labeling: list, initial_state, name: str ="unk
 	name : str, optional
 		Name of the model.
 		Default is "unknow_CTMC"
+	synchronous_transitions: list, optional.
+		This is useful only for synchronously composing this CTMC with
+		another one.
+		List of (source_state <int>, action <str>, dest_state <int>, rate <float>).
+		Default is an empty list.
 	
 	Returns
 	-------
 	CTMC
-		the CTMC describes by `transitions`, `labeling`, and `initial_state`.
+		the CTMC describes by `transitions`, `labelling`, and `initial_state`.
 	
 	Examples
 	--------
@@ -465,24 +458,26 @@ def createCTMC(transitions: list, labeling: list, initial_state, name: str ="unk
 	s1 -> s0 : lambda = 0.3
 	s1 -> s1 : lambda = 0.2
 	"""
-	if 'init' in labeling:
+	if 'init' in labelling:
 		msg =  "The label 'init' cannot be used: it is reserved for initial states."
 		raise SyntaxError(msg)
 	
-	states = list(set([i[0] for i in transitions]+[i[1] for i in transitions]))
+	states = [i[0] for i in transitions]+[i[1] for i in transitions]
+	states +=[i[0] for i in synchronous_transitions]+[i[2] for i in synchronous_transitions]
+	states = list(set(states))
 	states.sort()
 	nb_states = len(states)
 	
-	if nb_states > len(labeling):
-		raise ValueError("All states are not labelled (the labeling list is too small).")
-	elif nb_states < len(labeling):
-		print("WARNING: the labeling list is bigger than the number of states")
+	if nb_states > len(labelling):
+		raise ValueError("All states are not labelled (the labelling list is too small).")
+	elif nb_states < len(labelling):
+		print("WARNING: the labelling list is bigger than the number of states")
 
 	res = zeros((nb_states,nb_states))
 	for t in transitions:
 		res[states.index(t[0])][states.index(t[1])] = t[2]
 	
-	labeling.append('init')
+	labelling.append('init')
 	res = vstack((res,zeros(len(res))))
 	res = hstack((res,zeros(len(res))[:,newaxis]))
 	if type(initial_state) == int:
@@ -491,5 +486,154 @@ def createCTMC(transitions: list, labeling: list, initial_state, name: str ="unk
 		if type(initial_state) == ndarray:
 			initial_state = initial_state.tolist()
 		res[-1] = array(initial_state+[0.0])
+	return CTMC(res, labelling, name,synchronous_transitions)
+
+
+def synchronousCompositionCTMCs(m1: CTMC, m2: CTMC, name: str = "unknown_composition") -> PCTMC:
+	"""
+	Returns the synchornous compotision of `m1` and `m2`.
+	The output model is a PCTMC since some parameters are shared
+	by multiple transitions.
+
+	Parameters
+	----------
+	m1 : CTMC
+		First CTMC to compose with.
+	m2 : CTMC
+		Second CTMC to compose with.
+
+	Returns
+	-------
+	PCTMC
+		Synchronous composition of `m1` and `m2`.
+	"""
+	m1_init = [i for i,li in enumerate(m1.labelling) if li == 'init']
+	m2_init = [i for i,li in enumerate(m2.labelling) if li == 'init']
+	m1_nb_states = m1.nb_states - len(m1_init)
+	m2_nb_states = m2.nb_states - len(m2_init)
+	nb_states = m1_nb_states * m2_nb_states	
+	m1_sids = [i-m1.labelling[:i].count("init") for i,li in enumerate(m1.labelling) if li != 'init']
+	m2_sids = [i-m1.labelling[:i].count("init") for i,li in enumerate(m2.labelling) if li != 'init']
+
+	matrix= zeros((nb_states,nb_states),dtype='uint8')
+	labelling = []
+	p_v = {}
+	p_str = []
+	p_i = []
+	t_e = [sympify('0.0')]
+
+	def get_state_index(s1,s2):
+		s2 = m2_sids.index(s2)
+		s1 = m1_sids.index(s1)
+		return s1*m2_nb_states+s2
+
+	def add_in_matrix(trans,s1,s2,model,add_index=[]):
+		if model == 1:
+			for i in m2_sids:
+				x,y = get_state_index(s1,i),get_state_index(s2,i)
+				matrix[x,y] = trans
+				for ind in add_index:
+					p_i[ind].append([x,y])
+		elif model == 2:
+			for i in m1_sids:
+				x,y = get_state_index(i,s1),get_state_index(i,s2)
+				matrix[x,y] = trans
+				for ind in add_index:
+					p_i[ind].append([x,y])
+
+	for s1 in m1_sids: # labelling
+		for s2 in m2_sids:
+			labelling.append(m1.labelling[s1]+','+m2.labelling[s2])
+
+	for i in m1_sids: # m1 transitions
+		for j in m1_sids:
+			if m1.matrix[i,j] != 0.0:
+				p_str.append("p_"+str(i)+'_'+str(j))
+				p_v["p_"+str(i)+'_'+str(j)] = m1.matrix[i,j]
+				p_i.append([])
+				t_e.append(sympify(p_str[-1]))
+				add_in_matrix(len(t_e)-1,s1=i,s2=j,model=1,add_index=[len(p_i)-1])
+	for i in m2_sids: # m2 transitions
+		for j in m2_sids:
+			if m2.matrix[i,j] != 0.0:
+				p_str.append("q_"+str(i)+'_'+str(j))
+				p_v["q_"+str(i)+'_'+str(j)] = m2.matrix[i,j]
+				p_i.append([])
+				t_e.append(sympify(p_str[-1]))
+				add_in_matrix(len(t_e)-1,s1=i,s2=j,model=2,add_index=[len(p_i)-1])
+
+	for i in m1_sids: # self loops
+		if m1.matrix[i,i] != 0.0:
+			for j in m2_sids:
+				if m2.matrix[j,j] != 0.0:
+					matrix[get_state_index(i,j),get_state_index(i,j)] = len(t_e)
+					t_e.append(sympify("p"+str(i)+'_'+str(i)+'+q'+str(j)+'_'+str(j)))
+
+	for si,ai,di,pi in m1.synchronous_transitions: # synchronous transitions
+		for sj,aj,dj,pj in m2.synchronous_transitions:
+			if ai == aj:
+				t_e.append(t_e[matrix[get_state_index(si,sj),get_state_index(di,dj)]]+sympify(pi*pj))
+				matrix[get_state_index(si,sj),get_state_index(di,dj)] = len(t_e)-1
+	
+	labelling.append('init')
+	matrix = vstack((matrix,zeros(nb_states,dtype='uint8')))
+	matrix = hstack((matrix,zeros(nb_states+1,dtype='uint8')[:,newaxis]))
+	m1_init_trans = zeros(m1_nb_states)
+	for i in m1_init:
+		tmp = [m1.matrix[i][j] for j in m1_sids]
+		for j in range(m1_nb_states):
+			if m1_init_trans[j]*tmp[j]>0.0:
+				m1_init_trans[j] *= tmp[j]
+			else:
+				m1_init_trans[j] += tmp[j]
+	m2_init_trans = zeros(m2_nb_states)
+	for i in m2_init:
+		tmp = [m2.matrix[i][j] for j in m2_sids]
+		for j in range(m2_nb_states):
+			if m2_init_trans[j]*tmp[j]>0.0:
+				m2_init_trans[j] *= tmp[j]
+			else:
+				m2_init_trans[j] += tmp[j]
+	for i,si in enumerate(m1_init_trans):
+		if si > 0.0:
+			p_str.append("initp_"+str(i))
+			p_v["initp_"+str(i)] = si
+			p_i.append([])
+	for i,si in enumerate(m2_init_trans):
+		if si > 0.0:
+			p_str.append("initq_"+str(i))
+			p_v["initq_"+str(i)] = si
+			p_i.append([])
+	for i,si in enumerate(m1_init_trans):
+		for j,sj in enumerate(m2_init_trans):
+			if si*sj <= 0.0:
+				matrix[-1][get_state_index(i,j)] = 0
+			else:
+				matrix[-1][get_state_index(i,j)] = len(t_e)
+				t_e.append(sympify('initp_'+str(i)+'*initq_'+str(j)))
+				p_i[p_str.index('initp_'+str(i))].append([len(matrix)-1,get_state_index(i,j)])
+				p_i[p_str.index('initq_'+str(j))].append([len(matrix)-1,get_state_index(i,j)])
+	
+	i = 0
+	while i < len(matrix): # removing unreachable states
+		if (matrix.T[i] == 0).all() == True and labelling[i] != 'init':
+			nb_states -= 1
+			matrix = delete(matrix,i,0)
+			matrix = delete(matrix,i,1)
+			labelling = labelling[:i]+labelling[i+1:]
+			for j in range(len(p_i)):
+				k = 0
+				while k < len(p_i[j]):
+					if p_i[j][k][0] == i or p_i[j][k][1] == i:
+						p_i[j].remove(p_i[j][k])
+					else:
+						if p_i[j][k][0] > i:
+							p_i[j][k][0] -= 1
+						if p_i[j][k][1] > i:
+							p_i[j][k][1] -= 1
+						k += 1
+			i = -1
+		i += 1
+	
+	return PCTMC(matrix, labelling, t_e, p_v, p_i, p_str, name)
 
-	return CTMC(res, labeling, name)
