@@ -1,11 +1,13 @@
-from .MC import *
+from .MC import MC
 from math import sqrt, log
 from ..base.Set import Set
 from ..base.tools import normalize
+from numpy import zeros
+
+
 class Alergia:
 	"""
-	class for general ALERGIA algorithm on MC.
-	This algorithm is described here:
+	Class for general ALERGIA algorithm described here:
 	https://grfia.dlsi.ua.es/repositori/grfia/pubs/57/icgi1994.pdf
 	"""
 
@@ -33,7 +35,7 @@ class Alergia:
 			self.alphabet = alphabet
 		self.createPTA(traces)
 	
-	def createPTA(self,traces: Set) -> MC:
+	def createPTA(self,traces: Set):
 		"""
 		Create a PTA from ``traces``.
 
@@ -85,7 +87,8 @@ class Alergia:
 					self.states_transitions[-1][2].append(self.states_lbl[s2][-1])
 					s2 += 1
 
-	def fit(self,traces: Set,alpha: float=0.1,alphabet: list=None, stormpy_output: bool = True):
+	def fit(self,traces: Set,alpha: float=0.1,alphabet: list=None,
+	 		stormpy_output: bool = True, output_file_prism: str = None):
 		"""
 		Fits a MC according to ``traces``.
 
@@ -101,6 +104,11 @@ class Alergia:
 		stormpy_output: bool, optional
 			If set to True the output model will be a Stormpy sparse model.
 			Default is True.
+		output_file_prism : str, optional
+			If set, the output model will be saved in a prism file at this
+			location. Otherwise the output model will not be saved.
+			This parameter is ignored if the model under learning is a HMM
+			or a GoHMM.
 
 		Returns
 		-------
@@ -130,6 +138,9 @@ class Alergia:
 			print("WARNING: stormpy not found. The output model will not be a stormpy sparse model")
 			stormpy_output = False
 		
+		if output_file_prism:
+			self.h.savePrism(output_file_prism)
+
 		if stormpy_output:
 			return jajapyModeltoStormpy(m)
 		else:
@@ -237,7 +248,7 @@ class Alergia:
 		states = [j for j in range(len(self.states_transitions)) if self.states_lbl[j] != None]
 		nb_init= len(states)
 		# states[x] = y : y->index alergia x->index jajapy
-		labeling = [None for s in states]
+		labelling = [None for s in states]
 		transitions = []
 
 		for s in range(nb_init):
@@ -247,25 +258,25 @@ class Alergia:
 			o = self.states_transitions[ai][2]
 			d = [states.index(dest) for dest in self.states_transitions[ai][1]]
 			for obs, dest, i in zip(o,d,range(len(d))):
-				if labeling[dest] == None:
-					labeling[dest] = obs
-				elif labeling[dest] != obs:
+				if labelling[dest] == None:
+					labelling[dest] = obs
+				elif labelling[dest] != obs:
 					d[i] = len(states)
-					labeling.append(obs)
+					labelling.append(obs)
 					states.append(states[dest])
 			transitions.append([(s,d[i],p[i]) for i in range(len(p))])
 				
 		for s in range(nb_init,len(states)):
 			transitions.append([(s,i[1],i[2]) for i in transitions[states.index(states[s])]])
 		
-		for i,j in enumerate(labeling):
+		for i,j in enumerate(labelling):
 			if j == None:
-				labeling[i] = ''
-				initial_state = i
-		
-		trans = []
-		for i in transitions:
-			trans += i
+				labelling[i] = 'init'
+				
+		trans = zeros((len(states),len(states)))
+		for j in transitions:
+			for i in j:
+				trans[i[0],i[1]] = i[2]
 
-		return createMC(trans,labeling,initial_state)
+		return MC(trans,labelling)
 		
