@@ -1,11 +1,10 @@
 from ..base.tools import resolveRandom, randomProbabilities, checkProbabilities
-from ..base.Model import Model
+from ..base.Base_MC import *
 from ast import literal_eval
-from numpy import ndarray, array, zeros, vstack, hstack, newaxis, append
-from random import choices
+from numpy import ndarray, array, zeros, vstack, hstack, newaxis, append, where
 
-class MC(Model):
-	def __init__(self, matrix: ndarray, labeling: list, name: str ="unknown_MC") -> None:
+class MC(Base_MC):
+	def __init__(self, matrix: ndarray, labelling: list, name: str ="unknown_MC") -> None:
 		"""
 		Creates an MC.
 
@@ -15,70 +14,22 @@ class MC(Model):
 			A (N x N) ndarray (with N the nb of states).
 			Represents the transition matrix.
 			`matrix[s1][s2]` is the probability of moving from `s1` to `s2`.
-		labeling: list of str
+		labelling: list of str
 			A list of N observations (with N the nb of states).
-			If `labeling[s] == o` then state of ID `s` is labelled by `o`.
+			If `labelling[s] == o` then state of ID `s` is labelled by `o`.
 			Each state has exactly one label.
 		name : str, optional
 			Name of the model.
 			Default is "unknow_MC"
-		"""
-		self.labeling = labeling
-		self.alphabet = list(set(labeling))
-		
-		if not 'init' in self.labeling:
-			msg = "No initial state given: at least one"
-			msg += " state should be labelled by 'init'."
-			raise ValueError(msg)
-		initial_state = [1.0/self.labeling.count("init") if i=='init' else 0.0 for i in self.labeling]
-
-		super().__init__(matrix,initial_state,name)
-		if len(self.labeling) != self.nb_states:
-			raise ValueError("The length of labeling ("+str(len(labeling))+") is not equal to the number of states("+str(self.nb_states)+")")
+		"""	
+		super().__init__(matrix,labelling,name)
+		self.model_type = MC_ID
 		for i in range(self.nb_states):
 			if not checkProbabilities(matrix[i]):
 				msg = "The probability to take a transition from state "
 				msg+= str(i)+" should be 1.0, here it's "+str(matrix[i].sum())
 				raise ValueError(msg)
-	
-	def getLabel(self,state: int) -> str:
-		"""
-		Returns the label of `state`.
-
-		Parameters
-		----------
-		state : int
-			a state ID
-
-		Returns
-		-------
-		str
-			a label
-
-		Example
-		-------
-		>>> model.getLabel(2)
-		'Label-of-state-2'
-		"""
-		self._checkStateIndex(state)
-		return self.labeling[state]
-	
-	def getAlphabet(self) -> list:
-		"""
-		Returns the alphabet of this model.
-
-		Returns
-		-------
-		list of str
-			The alphabet of this model
-		
-		Example
-		-------
-		>>> model.getAlphabet()
-		['a','b','c','d','done']
-		"""
-		return self.alphabet
-		
+			
 	def tau(self,s1: int, s2: int, obs: str) -> float:
 		"""
 		Returns the probability of moving from state `s1` to `s2` seeing label `obs`.
@@ -111,7 +62,7 @@ class MC(Model):
 		"""
 		self._checkStateIndex(s1)
 		self._checkStateIndex(s2)
-		if obs != self.labeling[s1]:
+		if obs != self.labelling[s1]:
 			return 0.0
 		return self.matrix[s1][s2]
 	
@@ -140,6 +91,11 @@ class MC(Model):
 		self._checkStateIndex(s2)
 		return self.matrix[s1][s2]
 
+	def b(self,state:int, label:str):
+		if self.getLabel(state) == label:
+			return 1.0
+		return 0.0
+
 	def next(self,state: int) -> tuple:
 		"""
 		Return a state-observation pair according to the distributions described by `matrix`
@@ -165,7 +121,7 @@ class MC(Model):
 		0.4
 		"""
 		c = resolveRandom(self.matrix[state])
-		return (c, self.labeling[state])
+		return (c, self.labelling[state])
 
 	def run(self, number_steps: int, current: int = -1) -> list:
 		"""
@@ -194,10 +150,10 @@ class MC(Model):
 			output.append(symbol)
 			current = next_state
 		
-		output.append(self.labeling[current])
+		output.append(self.labelling[current])
 		return output
 
-	def save(self, file_path:str):
+	def save(self, file_path:str) -> None:
 		"""Save the model into a text file.
 
 		Parameters
@@ -211,16 +167,37 @@ class MC(Model):
 		"""
 		f = open(file_path, 'w')
 		f.write("MC\n")
-		f.write(str(self.labeling))
-		f.write('\n')
 		super()._save(f)
 
 	def _stateToString(self,state:int) -> str:
-		res = "----STATE "+str(state)+"--"+self.labeling[state]+"----\n"
+		res = "----STATE "+str(state)+"--"+self.labelling[state]+"----\n"
 		for j in range(len(self.matrix[state])):
 			if self.matrix[state][j] > 0.0001:
-				res += "s"+str(state)+" -> s"+str(j)+" : "+str(self.matrix[state][j])+'\n'
+				res += "s"+str(state)+" -> s"+str(j)+" : "+str(round(self.matrix[state][j],5))+'\n'
 		return res
+	
+	def savePrism(self,file_path:str) -> None:
+		"""
+		Save this model into `file_path` in the Prism format.
+
+		Parameters
+		----------
+		file_path : str
+			Path of the output file.
+		"""
+		f = open(file_path,'w')
+		f.write("dtmc\n\n")
+		super()._savePrism(f)
+	
+	def _savePrismMatrix(self,f) -> None:
+		for s1 in range(self.nb_states):
+			if (self.matrix[s1]!=0).any():
+				res = '\t[] s='+str(s1)+' ->'
+				for s2 in where(self.matrix[s1] != 0.0)[0]:
+					res += ' '+str(self.matrix[s1,s2])+":(s'="+str(s2)+") +"
+				res = res[:-2]+';\n'
+				f.write(res)
+				
 
 def loadMC(file_path: str) -> MC:
 	"""
@@ -245,15 +222,15 @@ def loadMC(file_path: str) -> MC:
 	if l != "MC":
 		msg = "ERROR: this file doesn't describe an MC: it describes a "+l
 		raise ValueError(msg)
-	labeling = literal_eval(f.readline()[:-1])
+	labelling = literal_eval(f.readline()[:-1])
 	name = f.readline()[:-1]
 	initial_state = array(literal_eval(f.readline()[:-1]))
 	matrix = literal_eval(f.readline()[:-1])
 	matrix = array(matrix)
 	f.close()
-	return MC(matrix, labeling, name)
+	return MC(matrix, labelling, name)
 
-def MC_random(nb_states: int, alphabet: list, random_initial_state: bool=True) -> MC:
+def MC_random(nb_states: int, labelling: list, random_initial_state: bool=True) -> MC:
 	"""
 	Generate a random MC.
 
@@ -261,7 +238,7 @@ def MC_random(nb_states: int, alphabet: list, random_initial_state: bool=True) -
 	----------
 	number_states : int
 		Number of states.
-	alphabet : list of str
+	labelling : list of str
 		List of observations.
 	random_initial_state: bool, optional
 		If set to True we will start in each state with a random probability,
@@ -288,29 +265,19 @@ def MC_random(nb_states: int, alphabet: list, random_initial_state: bool=True) -
 	----STATE 2--init----
 	s2 -> s0 : 1.0
 	"""
-	if 'init' in alphabet:
-		msg =  "The label 'init' cannot be used: it is reserved for initial states."
-		raise SyntaxError(msg)
-
-	if nb_states < len(alphabet):
-		print("WARNING: the size of the alphabet is higher than the",end=" ")
-		print("number of states. Some labels will not be assigned to",end=" ")
-		print("any states.")
-	
-	labeling = alphabet[:min(len(alphabet),nb_states)] + choices(alphabet,k=nb_states-len(alphabet))
 	matrix = []
 	for _ in range(nb_states):
 		matrix.append(append(randomProbabilities(nb_states),0.0))
 
-	labeling.append("init")
 	if random_initial_state:
 		matrix.append(append(randomProbabilities(nb_states),0.0))
 	else:
 		matrix.append(array([1.0]+[0.0 for i in range(nb_states)]))
 	matrix = array(matrix)
-	return MC(matrix, labeling,"MC_random_"+str(nb_states)+"_states")
+	labelling = labelsForRandomModel(nb_states,labelling)
+	return MC(matrix, labelling,"MC_random_"+str(nb_states)+"_states")
 
-def createMC(transitions: list, labeling: list, initial_state, name: str ="unknown_MC") -> MC:
+def createMC(transitions: list, labelling: list, initial_state, name: str ="unknown_MC") -> MC:
 	"""
 	An user-friendly way to create a MC.
 
@@ -319,9 +286,9 @@ def createMC(transitions: list, labeling: list, initial_state, name: str ="unkno
 	transitions : [ list of tuples (int, int, float)]
 		Each tuple represents a transition as follow: 
 		(source state ID, destination state ID, probability).
-	labeling: list of str
+	labelling: list of str
 		A list of N observations (with N the nb of states).
-		If `labeling[s] == o` then state of ID `s` is labelled by `o`.
+		If `labelling[s] == o` then state of ID `s` is labelled by `o`.
 		Each state has exactly one label.
 	initial_state : int or list of float
 		Determine which state is the initial one (then it's the id of the
@@ -334,7 +301,7 @@ def createMC(transitions: list, labeling: list, initial_state, name: str ="unkno
 	Returns
 	-------
 	MC
-		the MC describes by `transitions`, `labeling`, and `initial_state`.
+		the MC describes by `transitions`, `labelling`, and `initial_state`.
 	
 	Examples
 	--------
@@ -350,7 +317,7 @@ def createMC(transitions: list, labeling: list, initial_state, name: str ="unkno
 	----STATE 2--init----
 	s2 -> s0 : 1.0
 	"""
-	if 'init' in labeling:
+	if 'init' in labelling:
 		msg =  "The label 'init' cannot be used: it is reserved for initial states."
 		raise SyntaxError(msg)
 	
@@ -358,16 +325,16 @@ def createMC(transitions: list, labeling: list, initial_state, name: str ="unkno
 	states.sort()
 	nb_states = len(states)
 	
-	if nb_states > len(labeling):
-		raise ValueError("All states are not labelled (the labeling list is too small).")
-	elif nb_states < len(labeling):
-		print("WARNING: the labeling list is bigger than the number of states")
+	if nb_states > len(labelling):
+		raise ValueError("All states are not labelled (the labelling list is too small).")
+	elif nb_states < len(labelling):
+		print("WARNING: the labelling list is bigger than the number of states")
 
 	res = zeros((nb_states,nb_states))
 	for t in transitions:
 		res[states.index(t[0])][states.index(t[1])] = t[2]
 	
-	labeling.append('init')
+	labelling.append('init')
 	res = vstack((res,zeros(len(res))))
 	res = hstack((res,zeros(len(res))[:,newaxis]))
 	if type(initial_state) == int:
@@ -377,4 +344,4 @@ def createMC(transitions: list, labeling: list, initial_state, name: str ="unkno
 			initial_state = initial_state.tolist()
 		res[-1] = array(initial_state+[0.0])
 
-	return MC(res, labeling, name)
+	return MC(res, labelling, name)
